from marinetrafficapi.query_params import QueryParams


class VI03QueryParams(QueryParams):
    """Query params params for VI03 API call."""

    params = {
        # The MarineTraffic ID of the port you wish to put as starting point
        # (found on the URL of the respective Port page) or port UN/LOCODE.
        'port_start_id': 'port_start_id',

        # The latitude of the starting point
        'latitude': 'LAT',

        # The longitude of the starting point
        'longitude': 'LON',

        # The Maritime Mobile Service Identity (MMSI) of the vessel you wish to track.
        'mmsi': 'mmsi',

        # The International Maritime Organization (IMO) number of the vessel you wish to track.
        'imo': 'imo',

        # A uniquely assigned ID by MarineTraffic for the subject vessel.
        'ship_id': 'shipid',

        # The MarineTraffic ID of the port you wish to put as target port
        # (found on the URL of the respective Port page) or port UN/LOCODE.
        'port_target_id': 'port_target_id',

        # Use 1 to search for all available routes regarding the specific journey
        # or 0 to not include alternative routes
        'include_alternatives': 'includealternatives',

        # Use 1 in order to receive routes which include inland waterways
        # or 0 to not include alternative routes
        'include_in_land': 'includeinland'
    }



from marinetrafficapi.query_params import QueryParams


class PS07QueryParams(QueryParams):
    """Query params for PS02 API call."""

    params = {
        # The maximum age, in minutes, of the returned positions.
        # Maximum value for terrestrial coverage is 60.
        # Maximum value for satellite coverage is 180.
        'time_span': 'timespan',

        # The Maritime Mobile Service Identity (MMSI) of the vessel you wish to track.
        'mmsi': 'mmsi',

        # The International Maritime Organization (IMO) number of the vessel you wish to track.
        'imo': 'imo',

        # A uniquely assigned ID by MarineTraffic for the subject vessel
        'shipid': 'shipid'
    }



import requests
from typing import TYPE_CHECKING

from marinetrafficapi.debug import Debug
from marinetrafficapi.formatter import FormatterFactory
from marinetrafficapi.exceptions import MarineTrafficRequestApiException
from marinetrafficapi.reponse import Response

if TYPE_CHECKING:
    from marinetrafficapi.client import Client


def bind_request(**request_data) -> 'callable':
    """Binds request class to client property, dynamically."""

    class ApiRequest(object):
        """Request class. Does the actual API request."""

        model = request_data.get('model')
        api_path = request_data.get('api_path')
        method = request_data.get('method', 'GET')
        query_parameters = request_data.get('query_parameters')
        default_parameters = request_data.get('default_parameters')
        fake_response_path = request_data.get('fake_response_path')

        def __init__(self, client: 'Client', debug: 'Debug',
                     *path_params, **query_params):
            client.request = self

            self.url = None
            self.debug = debug
            self.client = client
            self.parameters = {'query': {}, 'path': []}

            self._timeout = 5

            self._set_parameters(*path_params, **query_params)

        def _set_parameters(self, *path_params, **query_params) -> None:
            """
            Prepares the list of query parameters
            :path_params: list of path parameters
            :query_params: dict of query parameters
            :return: None
            """

            # take timeout
            try:
                self._timeout = int(query_params.get('timeout', self._timeout))
            except ValueError:
                pass
            try:
                del query_params['timeout']
            except KeyError:
                pass

            # set default API call params
            for key, value in self.default_parameters.items():
                self.parameters['query'][key] = value  # str(value).encode('utf-8')

            # set API call params defined during the "call" invocation
            for key, value in query_params.items():
                if value is None:
                    continue

                query_params = self.query_parameters.get_params()

                if key in query_params.values():
                    self.parameters['query'][key] = value  # str(value).encode('utf-8')

                elif key in query_params.keys():
                    self.parameters['query'][query_params[key]] = \
                        value  # str(value).encode('utf-8')

            # transform all True and False param to 1 and 0
            for key, value in self.parameters['query'].items():
                if value is True:
                    self.parameters['query'][key] = '1'
                if value is False:
                    self.parameters['query'][key] = '0'

            # set optional url path params
            for value in path_params:
                self.parameters['path'].append(value.encode('utf-8'))

        def _prepare_url(self) -> str:
            """
            Prepares url and query parameters for the request
            :return: URL
            """

            url_parts = {
                'protocol': self.client.protocol,
                'base_url': self.client.base_url,
                'base_path': self.client.base_path,
                'api_path': self.api_path,
                'api_key': self.client.api_key
            }
            url = '{protocol}://{base_url}{base_path}{api_path}/{api_key}'\
                .format(**url_parts)

            url_parts = self.parameters['path']
            url_parts.insert(0, url)

            url = '/'.join([part if type(part) == str else
                            part.decode('utf-8') for part in url_parts])

            self.debug.ok('url', url)
            self.debug.ok('query_parameters', self.parameters['query'])

            url = '{}/{}'.format(url, '/'.join(['{}:{}'.format(key, value)
                                                for key, value in
                                                self.parameters['query'].items()]))
            self.debug.ok('final url', url)

            return url

        def _do_request(self, url: str) -> (int, dict):
            """
            Makes the request to Marine Traffic Api servers
            :url: Url for the request
            :return: Tuple with two elements, status code and content
            """

            if self.client.fake_response_path:
                with open(self.client.fake_response_path, 'r') as f:
                    return 200, f.read()

            elif self.method == 'GET':
                response = requests.get(url, timeout=self._timeout)
                self.debug.ok('response_object', response)
                return response.status_code, response.text

            else:
                # For future POST, PUT, DELETE requests
                return 404, {}

        def _process_response(self, status_code: int, response: str) -> 'Response':
            """
            Process response using models
            :status_code: Response status code
            :response: Content
            :return: Response object
            """

            formatter = FormatterFactory(self.parameters['query']['protocol'])\
                .get_formatter()

            response = Response(response, status_code, formatter, self)

            error_response = response.to_list
            if 'errors' in error_response:
                # error responses have status_code 200 instead of 5xx

                self.debug.error('status_code', status_code)
                self.debug.error('response', error_response)

                msg = 'Request errors: {}'.format(''.join(
                    ['code {}: {}'.format(error['code'], error['detail'])
                     for error in error_response['errors']]))

                raise MarineTrafficRequestApiException(msg)
            else:
                self.debug.ok('status_code', status_code)
                self.debug.ok('response', response.raw_data)

            return response

        def call(self) -> 'Response':
            """
            Makes the API call
            :return: Return value from self._process_response()
            """

            self.url = self._prepare_url()
            status_code, response = self._do_request(self.url)
            return self._process_response(status_code, response)

    def call(client, *path_params, **query_params) -> 'Response':
        """
        Binded method for API calls
        :path_params: list of path parameters
        :query_params: dict of query parameters
        :return: Return value from ApiRequest.call()
        """

        with Debug(client=client) as debug:
            request = ApiRequest(client, debug, *path_params, **query_params)
            return request.call()

    return call



from marinetrafficapi.models import Model
from marinetrafficapi.fields import NumberField, RealNumberField, DatetimeField


class VesselHistoricalPosition(Model):
    """Get all vessel historical positions for a specific period of time."""

    mmsi = NumberField(index='MMSI',
                       desc="Maritime Mobile Service Identity - a nine-digit number "
                            "sent in digital form over a radio frequency that identifies "
                            "the vessel's transmitter station")

    status = NumberField(index='STATUS',
                         desc="The AIS Navigational Status of the subject vessel as "
                              "input by the vessel's crew - more. There might be "
                              "discrepancies with the vessel's detail page when vessel "
                              "speed is near zero (0) knots.")

    speed = NumberField(index='SPEED',
                        desc="The speed (in knots x10) that the subject vessel is "
                             "reporting according to AIS transmissions")

    longitude = RealNumberField(index='LON',
                                desc="A geographic coordinate that specifies the "
                                     "east-west position of the vessel on the "
                                     "Earth's surface")

    latitude = RealNumberField(index='LAT',
                               desc="a geographic coordinate that specifies the "
                                    "north-south position of the vessel on the "
                                    "Earth's surface")

    course = NumberField(index='COURSE',
                         desc="The course (in degrees) that the subject vessel is "
                              "reporting according to AIS transmissions")

    heading = NumberField(index='HEADING',
                          desc="The heading (in degrees) that the subject vessel is "
                               "reporting according to AIS transmissions")

    timestamp = DatetimeField(index='TIMESTAMP',
                              desc="The date and time (in UTC) that the subject vessel's "
                                   "position was recorded by MarineTraffic",
                              format='%Y-%m-%dT%H:%M:%S')

    ship_id = NumberField(index='SHIP_ID',
                          desc="A uniquely assigned ID by MarineTraffic "
                               "for the subject vessel")

    wind_angle = NumberField(index='WIND_ANGLE',
                             desc="The current angle of the wind in the subject area "
                                  "(in degrees, compared to True North)")

    wind_speed = NumberField(index='WIND_SPEED',
                             desc="The current wind speed in the subject area (in knots)")

    wind_temp = NumberField(index='WIND_TEMP',
                            desc="The current temperature of the wind in the "
                                 "subject area (in Celsius degrees)")



from marinetrafficapi.query_params import QueryParams


class EV01QueryParams(QueryParams):
    """Query params for EV01 API call."""

    params = {
        # The maximum age, in minutes, of the returned port calls. Maximum value is 2880.
        'time_span': 'timespan',

        # If used with the value extended,
        # the response includes voyage related data since the previous port call.
        'msg_type': 'msgtype',

        # Response type. Use one of the following: xml, csv, json, jsono (object)
        'protocol': 'protocol',

        # The MarineTraffic ID of the port you wish to monitor (found on the URL
        # of the respective Port page) or port UN/LOCODE.
        'port_id': 'portid',

        # The Maritime Mobile Service Identity (MMSI) of the vessel you wish to track.
        'mmsi': 'mmsi',

        # The International Maritime Organization (IMO) number of the vessel you wish to track.
        'imo': 'imo',

        # A uniquely assigned ID by MarineTraffic for the subject vessel.
        'ship_id': 'shipid',

        # Use 0 to only receive arrivals or 1 to only receive departures.
        # If not used, the response will include both.
        'move_type': 'movetype',

        # Use 1 to exclude vessels in transit
        'exclude_intransit': 'exclude_intransit',

        # Portcalls between fromdate and todate.
        # Date format: YYYY-MM-DD HH:MM
        'from_date': 'fromdate',
        'to_date': 'todate',

        # Data filter: minimum DWT
        # applicable to IMO-having vessels
        'dwt_min': 'dwt_min',

        # Data filter: maximum DWT
        # applicable to IMO-having vessels
        'dwt_max': 'dwt_max',

        # Data filter: minimum GT
        # applicable to IMO-having vessels
        'gt_min': 'gt_min',

        # Data filter: maximum GT
        # applicable to IMO-having vessels
        'gt_max': 'gt_max'
    }



from marinetrafficapi.query_params import QueryParams


class PS02PS06QueryParams(QueryParams):
    """Query params for PS02 API call."""

    params = {
        # The maximum age, in minutes, of the returned positions.
        # Maximum value for terrestrial coverage is 60.
        # Maximum value for satellite coverage is 180.
        'time_span': 'timespan',

        # Data filter: Vessel type.
        # (2=Fishing / 4=High Speed Craft / 6=Passenger / 7=Cargo / 8=Tanker)
        'ship_type': 'shiptype',

        'min_latitude': 'MINLAT',
        'max_latitude': 'MAXLAT',
        'min_longitude': 'MINLON',
        'max_longitude': 'MAXLON'
    }



from marinetrafficapi.events.client import Events
from marinetrafficapi.voyage_info.client import VoyageInfo
from marinetrafficapi.vessels_positions.client import VesselPositions
from marinetrafficapi.exceptions import MarineTrafficRequestApiException


class Client(VesselPositions, VoyageInfo, Events):
    """Main API class. Sets all API calls."""

    base_url = 'services.marinetraffic.com/en'
    base_path = '/api'
    protocol = 'https'

    def __init__(self, api_key: str = None, debug: bool = False,
                 fake_response_path: str = None):
        if not api_key:
            raise MarineTrafficRequestApiException('API key missing!')

        self.request = None
        self.api_key = api_key
        self.debug = debug
        self.fake_response_path = fake_response_path



from marinetrafficapi.bind import bind_request

from marinetrafficapi.voyage_info.\
    VI03_port_distances_and_routes.models import PortDistanceAndRoute

from marinetrafficapi.voyage_info.\
    VI03_port_distances_and_routes.query_params import VI03QueryParams


class VoyageInfo:
    """Retrieve forecasted information for any vessel.
    Get ETA and voyage related information using one of these APIs."""

    # VI03 - Get all the available vessel routes and the respective
    # distances from point to port or port to port.
    port_distances_and_routes = bind_request(
        api_path='/exportroutes',
        model=PortDistanceAndRoute,
        query_parameters=VI03QueryParams,
        default_parameters={
            'msgtype': 'simple',
            'protocol': 'jsono'
        }
    )



from marinetrafficapi.models import Model
from marinetrafficapi.fields import NumberField, BooleanField, LinestringField


class PortDistanceAndRoute(Model):
    """Receive a list of available routes and distances
    from a specific point to a port or from port to port."""

    distance = NumberField(index='DISTANCE',
                           desc="The Distance (in NM) between the specified point "
                                "or port to the destination port")

    panama = BooleanField(index='PANAMA',
                          desc="Whether the vessel route passes via the Panama canal")

    suez = BooleanField(index='SUEZ',
                        desc="Whether the vessel route passes via the Suez canal")

    final_path = LinestringField(index='FINAL_PATH',
                                 desc="Route/Waypoints as Linestring "
                                      "Geometry in WKT - Well-Known Text")



from marinetrafficapi.bind import bind_request

from marinetrafficapi.events.\
    EV01_port_calls.models import PortCall
from marinetrafficapi.events.\
    EV01_port_calls.query_params import EV01QueryParams


class Events:
    """Retrieve event information of any vessel.
    Select which events, including port calls and
    departures, using any of these APIs."""

    # EV01 - Get detailed arrival and departure
    #        information for a port or vessel.
    port_calls = bind_request(
        api_path='/portcalls',
        model=PortCall,
        query_parameters=EV01QueryParams,
        default_parameters={
            'v': '4',
            'msgtype': 'simple',
            'protocol': 'jsono'
        }
    )



from marinetrafficapi.query_params import QueryParams


class PS01QueryParams(QueryParams):
    """Query params for PS01 API call."""

    params = {
        # The number of days, starting from the time of request and going backwards,
        # for which the response should look for position data. Maximum value is 190 (days).
        'days': 'days',

        # Date format: YYYY-MM-DD HH:MM:SS. If absent, current date will be selected.
        'date_from': 'fromdate',
        'date_to': 'todate',

        # Possible values:
        # hourly: In order to receive hourly vessel positions
        # daily: In order to receive daily vessel positions
        'period': 'period',

        # Define an area for which you wish to either get all the historical
        # vessels' positions or historical positions for a specific vessel.
        # Important! If you do not define a vessel, the maximum period for
        # which you can look back has a maximum period of one day.
        'min_latitude': 'MINLAT',
        'min_longitude': 'MINLON',
        'max_latitude': 'MAXLAT',
        'max_longitude': 'MAXLON',

        # The Maritime Mobile Service Identity (MMSI) of the vessel you wish to track.
        'mmsi': 'mmsi',

        # The International Maritime Organization (IMO) number of the vessel you wish to track.
        'imo': 'imo',

        # A uniquely assigned ID by MarineTraffic for the subject vessel.
        'ship_id': 'shipid',
    }



from datetime import datetime
from typing import Any, Union, List, TYPE_CHECKING, Tuple

if TYPE_CHECKING:
    from marinetrafficapi.models import Model


class Field:
    """Abstract field class."""

    def __init__(self, index: Union[int, str],
                 desc: str = None, **kwargs):
        self._index = index
        self._desc = desc
        self._kwargs = kwargs

        self.data = None

    def convert_item(self, model: 'Model') -> None:
        """Convert item to desired item type"""

        try:
            self.data = model.item[self._index]
        except IndexError:
            self.data = None
        except KeyError:
            self.data = None

        try:
            self.data = self._convert_field_item(
                self.data, **self._kwargs
            )
        except TypeError:
            self.data = None

    def _convert_field_item(self, data: Any, **kwargs) -> Any:
        """Actual converting."""


class NumberField(Field):
    """Converting item to integer."""

    def _convert_field_item(self, data: str, **kwargs) -> int:
        """Actual converting."""

        try:
            return int(data)
        except ValueError:
            return 0


class RealNumberField(Field):
    """Converting item to integer."""

    def _convert_field_item(self, data: str, **kwargs) -> float:
        """Actual converting."""

        return float(data)


class TextField(Field):
    """Converting item to string."""

    def _convert_field_item(self, data: str, **kwargs) -> str:
        """Actual converting."""

        return str(data)


class BooleanField(Field):
    """Converting item to boolean."""

    def _convert_field_item(self, data: str, **kwargs) -> \
            Union[bool, None]:
        """Actual converting."""

        if data == '1':
            return True
        elif data == '0':
            return False
        else:
            return None


class DatetimeField(Field):
    """Converting item to datetime object."""

    def _convert_field_item(self, data: str, **kwargs) -> \
            Union['datetime', str]:
        """Actual converting."""

        _format = kwargs.get('format')
        return datetime.strptime(data, _format)


class LinestringField(Field):
    """Converting item to list of integers."""

    def _convert_field_item(self, data: str, **kwargs) -> \
            List[Tuple[float]]:
        """Actual converting."""

        coordinates = []
        for coordinate in data[12:-1].split(','):
            coordinates.append(
                tuple(map(float, coordinate.strip().split(' '))))

        return coordinates



from marinetrafficapi.models import Model
from marinetrafficapi.fields import NumberField, BooleanField, DatetimeField, TextField


class PortCall(Model):
    """Get all vessel historical positions for a specific period of time."""

    mmsi = NumberField(index='MMSI',
                       desc="Maritime Mobile Service Identity - a nine-digit number "
                            "sent in digital form over a radio frequency that identifies "
                            "the vessel's transmitter station")

    ship_name = TextField(index='SHIPNAME',
                          desc="The Ship name of the subject vessel")

    load_status = NumberField(index='LOAD_STATUS',
                              desc="The load status of the subject vessel "
                                   "(0 : N/A, 1 : In Ballast, 2 : Partially Laden, "
                                   "3 : Fully Laden)")

    voyage_avg_speed = NumberField(index='VOYAGE_SPEED_AVG',
                                   desc="Average Maintained Speed (in knots x10) since "
                                        "the last Port Call while steaming at speed > 5 "
                                        "knots - Updated upon Arriva")

    voyage_max_speed = NumberField(index='VOYAGE_SPEED_MAX',
                                   desc="Maximum Recorded Speed (in knots x10) since "
                                        "the last Port Call - Updated upon Arrival")

    local_timestamp = DatetimeField(index='TIMESTAMP_LT',
                                    desc="The Date and Time (in Local Time) of the Port Call",
                                    format='%Y-%m-%dT%H:%M:%S')

    utc_timestamp = DatetimeField(index='TIMESTAMP_UTC',
                                  desc="The Date and Time (in UTC) of the Port Call",
                                  format='%Y-%m-%dT%H:%M:%S')

    ship_id = NumberField(index='SHIP_ID',
                          desc="A uniquely assigned ID by MarineTraffic "
                               "for the subject vessel")

    move_type = BooleanField(index='MOVE_TYPE',
                             desc="0 or 1 - specifies the Port Call type "
                                  "(0 = Arrival, 1 = Departure)")

    type_name = TextField(index='TYPE_NAME',
                          desc="The Type of the subject vessel")

    port_id = NumberField(index='PORT_ID',
                          desc="A uniquely assigned ID by MarineTraffic for the Current Port")

    port_name = TextField(index='PORT_NAME',
                          desc="The Type of the subject vessel")

    unlocode = TextField(index='UNLOCODE',
                         desc="A uniquely assigned ID by United Nations for the subject Port")

    draught = NumberField(index='DRAUGHT',
                          desc="The Draught (in metres x10) of the subject vessel "
                               "according to the AIS transmissions")

    port_operation = NumberField(index='PORT_OPERATION',
                                 desc="Indicates LOAD/DISCHARGE operations in the subject "
                                      "Port Call event (0 : N/A, 1 : load, 2 : discharge, "
                                      "3 : both, 4 : none)")

    in_transit = BooleanField(index='INTRANSIT',
                              desc="0 or 1 - specifies whether the vessel anchored in the "
                                   "Port or was In Transit (1 = In Transit)")

    distance_travelled = NumberField(index='DISTANCE_TRAVELLED',
                                     desc="The Distance (in NM) that the subject vessel "
                                          "has travelled since departing from Last Port")

    voyage_idle_time = NumberField(index='VOYAGE_IDLE_TIME_MINS',
                                   desc="Total time in minutes since the previous Port Call "
                                        "while the vessel stopped or moving at speed < 3 "
                                        "knots - Updated upon Arrival")

    elapsed_noanch = NumberField(index='ELAPSED_NOANCH',
                                 desc="Total time of Voyage since the previous Port Call "
                                      "except Anchorages and Canals - Updated upon Arrival")

    distance_leg = NumberField(index='DISTANCE_LEG',
                               desc="The distance (in NM) between the last two calls "
                                    "regardless of Port Type")

    comfleet_grouped_type = TextField(index='COMFLEET_GROUPEDTYPE',
                                      desc="The commercial market segment the "
                                           "subject vessel belongs to")

    ship_class = TextField(index='SHIPCLASS',
                           desc="The class of the subject vessel based on vessel type and size")



import csv
import ujson
from lxml import etree
from io import StringIO
from typing import AnyStr, Any, Dict, Union, Type, List


class Formatter:
    """Main formatter class."""

    def __init__(self):
        pass

    def _format(self, data: AnyStr) -> Any:
        """Format method in formatter classes."""

        return data

    def format(self, data: AnyStr) -> Any:
        """Main format method."""

        return self._format(data)

    def _to_list(self, data: AnyStr) -> List:
        """To list method in formatter classes."""

        return data

    def to_list(self, data: AnyStr) -> List:
        """Main to list method."""

        return self._to_list(data)


class Json(Formatter):
    """JSON data formatter class."""

    def _format(self, data: AnyStr) -> Dict:
        """Transform raw data from server into python native type."""

        return ujson.loads(data)


class Csv(Formatter):
    """CSV data formatter class."""

    def _format(self, data: AnyStr) -> Union[List, Dict]:
        """Transform raw data from server into python native type."""

        data = list(csv.reader(StringIO(data)))

        csv_data = []

        if data[0][0].startswith('ERROR_'):
            # errors are pure txt message,
            # nothing like csv.
            # trying to follow the same error message
            # structure from json error response.

            csv_data = {
                'errors': [
                    {
                        "code": data[0][0].split('_')[1].split('-')[0],
                        "detail": data[0][0].split('-')[1]
                    }
                ]
            }

        csv_keys = []
        for i, line in enumerate(data):
            line_data = {}
            if i == 0:
                csv_keys = [k.strip() for k in line]
            else:
                for j, key in enumerate(csv_keys):
                    line_data[key] = line[j]

                # sometimes number of delimiters in data part of
                # csv is bigger then in keys part, so we need
                # to concatenate the rest of the delimited parts
                # of data line.
                if len(csv_keys) < len(line):
                    line_data[csv_keys[-1]] = ','.join(line[len(csv_keys)-1:])

                csv_data.append(line_data)

        return csv_data


class Xml(Formatter):
    """XML data formatter class."""

    def _format(self, data: AnyStr) -> Union[List, Dict]:
        """Transform raw data from server into python native type."""

        data = etree.fromstring(data.encode('utf-8'))

        xml_data = []

        if list(data)[0].tag == 'STATUS':
            if list(list(data)[0])[0].tag == 'ERROR':
                # error message does not follow
                # same structure as data message.
                # trying to follow the same error message
                # structure from json error response.

                attribs = list(list(data)[0])[0].attrib
                xml_data = {
                    'errors': [
                        {
                            "code": attribs['CODE'],
                            "detail": attribs['DESCRIPTION']
                        }
                    ]
                }
        else:
            xml_data = [child.attrib for child in data]

        return xml_data


class FormatterFactory:
    """Formatter factory class."""

    formatters = {
        'json': Json,
        'jsono': Json,
        'xml': Xml,
        'csv': Csv
    }

    def __init__(self, name: str):
        self._name = name

    def get_formatter(self) -> Union[Type[Json], Type[Xml], Type[Csv], None]:
        """Get the formatter class using string key."""

        return FormatterFactory.formatters.get(self._name)



class QueryParams:
    """Query parameters for API calls."""

    params = {}

    default_params = {
        # simple or extended (extended cost more credits)
        'msg_type': 'msgtype',

        # Response type. Use one of the following: xml, csv, json, jsono (object)
        'protocol': 'protocol'
    }

    def __init__(self):
        pass

    @classmethod
    def get_params(cls):
        return {**cls.params, **QueryParams.default_params}



from marinetrafficapi.bind import bind_request

from marinetrafficapi.vessels_positions.\
    PS01_vessel_historical_track.models import VesselHistoricalPosition
from marinetrafficapi.vessels_positions.\
    PS02_PS06_vessel_positions.models import FleetVesselPosition

from marinetrafficapi.vessels_positions.\
    PS01_vessel_historical_track.query_params import PS01QueryParams
from marinetrafficapi.vessels_positions.\
    PS02_PS06_vessel_positions.query_params import PS02PS06QueryParams

from marinetrafficapi.vessels_positions.\
    PS07_single_vessel_positions.query_params import PS07QueryParams
from marinetrafficapi.vessels_positions.\
    PS07_single_vessel_positions.models import SingleVesselPosition


class VesselPositions:
    """Retrieve forecasted information for any vessel.
    Get ETA and voyage related information using one of these APIs."""

    # PS01 - Get all historical positions for one or more vessels over
    #        a period of time.
    vessel_historical_track = bind_request(
        api_path='/exportvesseltrack',
        model=VesselHistoricalPosition,
        query_parameters=PS01QueryParams,
        default_parameters={
            'v': '2',
            'msgtype': 'simple',
            'protocol': 'jsono'
        }
    )

    # PS02 - Get positional information for a set of predefined vessels.
    # PS03 - Monitor vessel activity for your MarineTraffic fleet(s)
    # PS04 - Monitor vessel activity in one or more ports of your interest
    # PS05 - Monitor vessel activity in an area of your interest
    # PS06 - Retrieve positions for vessels sailing in an area that
    #        you define each time you call the service
    fleet_vessel_positions = bind_request(
        api_path='/exportvessels',
        model=FleetVesselPosition,
        query_parameters=PS02PS06QueryParams,
        default_parameters={
            'v': '8',
            'msgtype': 'simple',
            'protocol': 'jsono'
        }
    )

    # PS07 - Get the latest available position or voyage information for
    #        a particular vessel.
    single_vessel_positions = bind_request(
        api_path='/exportvessel',
        model=SingleVesselPosition,
        query_parameters=PS07QueryParams,
        default_parameters={
            'v': '5',
            'msgtype': 'simple',
            'protocol': 'jsono'
        }
    )



from setuptools import setup, find_packages


def readme():
    with open('README.md') as f:
        return f.read()


setup(
    name='Marine Traffic API',
    version='0.6.4',

    description='Marine Traffic Client Api',
    long_description=readme(),
    long_description_content_type='text/markdown',

    url='https://github.com/amphinicy/marine-traffic-client-api',
    licence='MIT',

    author='Ivan Arar @ Amphinicy Technologies',
    author_email='ivan.arar@amphinicy.com',

    classifiers=[
        'Development Status :: 4 - Beta',
        'Intended Audience :: Developers',
        'Topic :: Software Development :: Build Tools',
        'License :: OSI Approved :: MIT License',
        'Programming Language :: Python :: 3',
    ],
    keywords='marine traffic, api, cruise, distance, port, vessel, track, fleet',

    packages=find_packages(),
    install_requires=[
        'click~=6.7',
        'lxml>=4.3.0',
        'ujson>=1.35',
        'requests~=2.20.0'
    ],

    project_urls={
        'Source': 'https://github.com/amphinicy/marine-traffic-client-api',
    },
)
